#! /usr/bin/env bash

#################
# Pass fimc's pid as param to the script. In case of fimc not running, pass 0.
#################

FIMC_PID=$1

AVAILABLE=0
VERIFIED=1
FAILED=2

## Keep below command and 'if' condition together.
## Any assignment operation also changes the value in $? varaiable.
AUDITCTL_OP=`auditctl -s 2> /dev/null`
if [ $? -ne 0 ]
then
	# auditctl utility not found. assume auditd is also not present on the system.
	if [[ $FIMC_PID == 0 ]]
	then
		#kill any existing fimc processes assuming they are orphans
		pkill -15 fimc 2> /dev/null
		exit $AVAILABLE
	else
		exit $VERIFIED
	fi
fi

AUDIT_CTL_PID=`auditctl -s 2> /dev/null | grep pid | head -1 | sed -r 's/.*pid[= ]([0-9]+).*/\1/'`
if [[ $AUDIT_CTL_PID == 0 ]]
then
	exit $AVAILABLE
fi

# compare both process id. If equal, fimc is having control of auditd.
if [[ "$AUDIT_CTL_PID" == "$FIMC_PID" ]]
then
	exit $VERIFIED
else # not matching
	if [ $FIMC_PID -eq 0 ] # auditctl present; fimc pid param = 0
	then
		cmdout=`pgrep -x fimc |grep $AUDIT_CTL_PID`
		if [[ $cmdout != "" ]] # some orphan fimc is holding audit channel
		then
			#kill all existing fimc processes assuming they are orphans
			pkill -15 fimc 2> /dev/null
			AUDIT_CTL_PID=`auditctl -s 2> /dev/null | grep pid | head -1 | sed -r 's/.*pid[= ]([0-9]+).*/\1/'`
			if [[ $AUDIT_CTL_PID == 0 ]]
			then
				exit $AVAILABLE
			else
				exit $FAILED
			fi				
		else # some 3rd party process is holding audit channel. Leave it as is.
			exit $FAILED
		fi
	else
		# Reaches here only if fimc had control of auditd when started but then somehow lost it.
		exit $FAILED
	fi
fi
