#!/bin/bash

AUDIT_CONF="/etc/audit/auditd.conf"
AUDIT_DISPATCHER_PLUGINS_DIR="/etc/audisp/plugins.d"
AUDIT_FIM_PLUGIN_CONF="${AUDIT_DISPATCHER_PLUGINS_DIR}/fim-plugin.conf"

SOURCE_CFG="/etc/qualys/cloud-agent/fim-plugin.cfg"
SOURCE_PLUGIN_CONF="/etc/qualys/cloud-agent/fim-plugin.conf"

FIM_POLICY_MOD="/usr/local/qualys/cloud-agent/bin/fim-plugin-policy.mod"
FIM_AUDISP_POLICY_TE="/usr/local/qualys/cloud-agent/bin/fim-audisp-policy.te"
FIM_AUDITD_POLICY_TE="/usr/local/qualys/cloud-agent/bin/fim-auditd-policy.te"
FIM_POLICY_PP="/usr/local/qualys/cloud-agent/bin/fim-plugin-policy.pp"

FIM_POLICY_NAME="fim-plugin-policy"
# To check auditd version is above 3.0 or not
# above 3.0 auditd version plugin configuration directory locations are changed.
AUDITD_MAJOR_VERSION="1"

if [ "$#" -ne 1 ]; then
	exit 1;
fi

## explictly adding /usr/sbin and /usr/bin in path and command used here and from these path.
export PATH=$PATH:/usr/sbin:/usr/bin

QUALYS_PID=$1;

if [ `whoami` != root ]; then
    echo "run as root or using sudo.Exiting." >&2
    exit 2
fi

#check auditctl exists or not
which auditctl 1> /dev/null 2>&1
if [ $? == 0 ]; then
	AUDITD_MAJOR_VERSION=`auditctl -v|awk -F " " '{print $3}'|sed -n 's/\([0-9]\+\).*/\1/p'`
fi

ensure_audisp(){

	if [ ! -x "$(which auditd)" ]; then
	  echo "auditd is not found in PATH=$PATH" >&2
	  exit 1
	fi

	#check that audit plugin conf folder exist
	if [ ! -d "$AUDIT_DISPATCHER_PLUGINS_DIR" ]; then
		echo "unable to find audit plugin folder $AUDIT_DISPATCHER_PLUGINS_DIR" >&2
		exit 1
	fi

	#check that audisp is the dispatcher selected in audit_conf	
	if [ -f "$AUDIT_CONF" ]; then
		#check that audisp is set as dispatcher
		CHKAUDISPD=` grep "dispatcher*" "$AUDIT_CONF" | grep audispd | awk -F"#" '{ print $1 }' ` 
		if [ -z "$CHKAUDISPD" ]; then
			echo "audispd is not set up as dispatcher in auditd.conf" >&2
			exit 1
		else
			echo "OK : audispd is set up as dispatcher in auditd.conf" 
		fi
	else
		echo "unable to find file $AUDIT_CONF" >&2
		exit 2
	fi
}

reload_auditd(){
	

	AUDITDPID=`pgrep auditd -x`
	if [ -z "$AUDITDPID" ]; then
		echo "auditd is not running" >&2
		service auditd start;
	else
		echo "auditd is running, reloading configuration"
		#kill -HUP "$AUDITDPID"
		service auditd restart 	
	fi


}

create_cfg(){

	if [ "$QUALYS_PID" != "X" ]
	then
		#> $DST_CFG
		#echo "netlink_pid=$QUALYS_PID" > $DST_CFG;	
		sed -i "s/netlink_pid=.*/netlink_pid=${QUALYS_PID}/" "$SOURCE_CFG"

	fi

}

load_fimplugin_policy(){

which sestatus;
if [ $? == 0 ]; then

selinuxStatus=`sestatus|grep "SELinux status"|awk -F" " '{print $3}'`;

if [ ! -z "$selinuxStatus" ]; then
	selinuxStatus=`echo $selinuxStatus|tr '[:upper:]' '[:lower:]'`
	if [ "$selinuxStatus" == "disabled" ]; then
		return 1;
	elif [ "$selinuxStatus" == "enabled" ]; then
		#check for current mode .
		selinuxStatus=`sestatus|grep "Current mode"|awk -F" " '{print $3}'`
		selinuxStatus=`echo $selinuxStatus|tr '[:upper:]' '[:lower:]'`
		if [ "$selinuxStatus" == "disabled" ]; then
			return 1;	
		fi
	fi

fi

which checkmodule;
if [ $? != 0 ]; then
        echo "checkmodule command not install.Please install checkpolicy or selinux-policy-default package." >&2;
        exit 1;
fi

which semodule_package;
if [ $? != 0 ]; then
        echo "semodule_package command not install.Please install policycoreutils-python or selinux-policy-default package." >&2;
        exit 1;
fi

which semodule;
if [ $? != 0 ]; then
        echo "semodule command not install.Please install policycoreutils or selinux-policy-default package." >&2;
        exit 1;
fi

	if [ "$AUDITD_MAJOR_VERSION" -ge 3 ] ; then
		FIM_POLICY_TE="$FIM_AUDITD_POLICY_TE";
	else
		FIM_POLICY_TE="$FIM_AUDISP_POLICY_TE";
	fi

	checkmodule -M -m -o $FIM_POLICY_MOD $FIM_POLICY_TE  > /dev/null 2>&1;
	semodule_package -o $FIM_POLICY_PP -m $FIM_POLICY_MOD > /dev/null 2>&1;

	policy_present=`semodule -l|grep $FIM_POLICY_NAME|wc -l` >/dev/null 2>&1;
	if [ $policy_present -gt 0 ]; then
		semodule -u $FIM_POLICY_PP > /dev/null 2>&1;
	else
		semodule -i $FIM_POLICY_PP  > /dev/null 2>&1;
	fi


	if [ -f "$FIM_POLICY_MOD" ]; then
		rm -rf $FIM_POLICY_MOD > /dev/null 2>&1;
	fi

	if [ -f "$FIM_POLICY_PP" ]; then
		rm -rf $FIM_POLICY_PP > /dev/null 2>&1;
	fi


fi


}
# If auditd version is greater than or equal to 3.0 then reconfigure Plugin directory
# location.
if [ "$AUDITD_MAJOR_VERSION" -ge 3 ] ; then
	# re-initialise auditd configuration directory for dispatcher.
	AUDIT_DISPATCHER_PLUGINS_DIR="/etc/audit/plugins.d"
	AUDIT_FIM_PLUGIN_CONF="${AUDIT_DISPATCHER_PLUGINS_DIR}/fim-plugin.conf"
else
	ensure_audisp
fi

#remove previous install if there and reload audisp or previous binary will be busy and prevent overwriting
#copy audisp plugin bnary
if [ -f "$AUDIT_CONF" ]; then
	#before reloading auditd .lets create configureation file.
	create_cfg	

	load_fimplugin_policy
	
	cp "$SOURCE_PLUGIN_CONF" "$AUDIT_FIM_PLUGIN_CONF"

	reload_auditd

	#leave time to plugin process to start properly
	sleep 1 
fi


echo "fim-plugin should be active"
exit 0;
